/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : UART0.c
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : Oobt@̃T|[g֐ł
* Creation Date: 2016/08/11
*******************************************************************************/

/*******************************************************************************
Includes
*******************************************************************************/
#include "r_cg_macrodriver.h"
#include "UART3.h"
#include "r_cg_userdefine.h"

#define init_bf u_uart3_init_bf
#define chk_status u_uart3_chk_status
#define get_data u_uart3_get_data
#define get_blk u_uart3_get_blk
#define g_rx_buff g_uart3_rx_buff
#define g_rx_rdpt g_uart3_rx_rdpt
#define g_rx_dtno g_uart3_rx_dtno
#define g_rx_status g_uart3_rx_status
#define uartx_callback_receivedata u_uart3_callback_receivedata
#define RX_BUFF_SIZE UART3_RX_BUFF_SIZE
#define RX_DTPT_MASK (UART3_RX_BUFF_SIZE - 1)

#if 0    /* Non-OS */
#define SRMKx(mask) do{ SRMK3 = mask; }while (0)
#define SRENx()     do{ SRMK3 = 0; }while (0)
#elif 1    /* FreeRTOS */
#define SRMKx(mask) do{ if (mask != 0) { taskENTER_CRITICAL(); } else { taskEXIT_CRITICAL(); } }while (0)
#define SRENx()     do{ SRMK3 = 0; }while (0)
#elif 0    /* SEGGER embOS */
#elif 0    /* Micrium MicroC/OS */
#elif 0    /* Renesas RI600V4 & RI600PX */
#endif

/*******************************************************************************
Pragma directive
*******************************************************************************/

/*******************************************************************************
Global variables and functions
*******************************************************************************/
volatile uint8_t g_rx_buff[RX_BUFF_SIZE];       /* MpOobt@     */
volatile uint8_t g_rx_rdpt;                     /* f[^̓ǂݏo|C^ */
volatile uint8_t g_rx_dtno;                     /* i[Ăf[^   */
volatile uint8_t g_rx_status;                   /* MXe[^XtO     
        bit7 : I[ot[EtO                                         */                                                                                                                          
/*******************************************************************************
* Function Name: init_bf
* Description  : Oobt@̃|C^
* Arguments    : None
* Return Value : None
*******************************************************************************/
void init_bf(void)
{

    SRMKx(1);                                   /* INTSRxƂ̔r       */
    g_rx_rdpt = 0x00;                           /* ǂݏo|C^   */
    g_rx_dtno = 0x00;                           /* f[^NA         */
    g_rx_status = 0x00;                         /* Xe[^XNA       */
    SRENx();                                    /* INTSRx̍ŏ̋       */
    SRMKx(0);                                   /* r䊮             */

}

/*******************************************************************************
* Function Name: chk_status
* Description  : Oobt@̏ԊmF
* Arguments    : None
* Return Value : Mf[^+Xe[^X
*               bit7 : I[ot[EtO
*               bit6,5 : 0,0
*               bit4-0 : f[^
*******************************************************************************/
uint8_t chk_status(void)
{
    uint8_t work;

    SRMKx(1);                                   /* INTSRxƂ̔r       */
    work = g_rx_status;                         /* Xe[^Xǂݏo       */
    g_rx_status = 0x00;                         /* Xe[^XNA       */
    work |= g_rx_dtno;                          /* f[^ǂݏo         */
    SRMKx(0);                                   /* r䊮             */
    return ( work );

}

/*******************************************************************************
* Function Name: get_data
* Description  : Oobt@1f[^ǂݏo
* Arguments    : None
* Return Value : f[^
*******************************************************************************/
uint8_t get_data(void)
{
    uint8_t work;

    if ( 0 == g_rx_dtno )
    {
        work = 0x00;                            /* f[^Ȃ΁C0x00   */
    }
    else
    {
        work = g_rx_buff[g_rx_rdpt];            /* f[^̓ǂݏo         */

        SRMKx(1);                               /* INTSRxƂ̔r       */
        g_rx_dtno--;
        g_rx_rdpt++;                            /* |C^XV             */
        g_rx_rdpt &= RX_DTPT_MASK;
        SRMKx(0);                               /* r䊮             */
    }

    return ( work );

}

/*******************************************************************************
* Function Name: get_blk
* Description  : Oobt@w肳ꂽf[^ǂݏoi]j
* Arguments    : i[|C^Cf[^
* Return Value : cf[^
*******************************************************************************/
uint8_t get_blk(uint8_t * const buff, uint8_t number)
{

    uint8_t work;
    uint8_t dtno;                               /* Oobt@f[^ */
    uint8_t cnt;                                /* ]pJE^           */
    uint8_t * gp_buff;                          /* ]p|C^           */

    work = number;                              /* vf[^Zbg     */
    dtno = g_rx_dtno;                           /* ϐvolatileȂ̂ŕێ */

    if ( ( 0 != number ) && ( 0 != dtno ) )
    {                               /* ]f[^ꍇ                 */

        gp_buff = buff;                         /* ]|C^ݒ     */

/* ---------------------------------------------------------------------------

    ]f[^Zo

----------------------------------------------------------------------------*/

        if ( number < dtno )
        {                           /* vf[^ʈȉ̏ꍇ           */
            cnt = number;                       /* ]f[^Zbg     */
            work = 0;                           /* cf[^0ɂ    */
        }
        else
        {                           /* vf[^ʈȏ̏ꍇ           */
            cnt = dtno;                    /* ]̓obt@̑S */
            work = number - dtno;          /* cf[^Zo       */
        }

/* ---------------------------------------------------------------------------

    f[^]

----------------------------------------------------------------------------*/

        for (  ; cnt > 0 ; cnt-- )
        {
            *gp_buff = g_rx_buff[g_rx_rdpt];    /* Oobt@]   */
            SRMKx(1);                           /* INTSRxƂ̔r       */
            g_rx_rdpt++;                        /* ǂݏo|C^XV   */
            g_rx_rdpt &= RX_DTPT_MASK;
            g_rx_dtno--;                        /* f[^|1            */
            SRMKx(0);                           /* INTSRxƂ̔rI   */
            gp_buff++;                          /* ]|C^XV       */

        }
    

    }

    return( work );

}

/*******************************************************************************
* Function Name: uartx_callback_receivedata
* Description  : This function is a callback function when UARTx receives a data.
* Arguments    : rx_data -
*                    receive data
* Return Value : None
*******************************************************************************/
void uartx_callback_receivedata(uint16_t rx_data)
{
    /* MpOobt@ */

    uint8_t  setptr;

    if ( g_rx_dtno < RX_BUFF_SIZE )
    {                           /* obt@ɋ󂫂                       */
        setptr = ((g_rx_rdpt + g_rx_dtno) & RX_DTPT_MASK);  /* ݃|C^ */
        g_rx_buff[setptr] = rx_data;    /* Mf[^obt@Ɋi[       */
        g_rx_dtno++;
    }
    else
    {                           /* obt@t̏                   */
        g_rx_status = 0x80;             /* I[ot[EtOZbg     */
    }

}
